@@ -9,6 +9,7 @@ from zope.sqlalchemy import ZopeTransactionExtension
 from pyramid.authentication import AuthTktAuthenticationPolicy
 from pyramid.authorization import ACLAuthorizationPolicy
 from pyramid.config import Configurator
+from pyramid.events import BeforeRender
 from pyramid.session import UnencryptedCookieSessionFactoryConfig
 from pyramid.threadlocal import get_current_registry
 from pyramid.util import DottedNameResolver
@@ -42,6 +43,7 @@ conf_defaults = {
     'kotti.templates.base_css': 'kotti:static/base.css',
     'kotti.templates.view_css': 'kotti:static/view.css',
     'kotti.templates.edit_css': 'kotti:static/edit.css',
+    'kotti.templates.api': 'kotti.views.util.TemplateAPI',
     'kotti.configurators': '',
     'kotti.base_includes': 'kotti.events kotti.views.view kotti.views.edit kotti.views.login kotti.views.users kotti.views.site_setup kotti.views.slots',
     'kotti.includes': '',
@@ -57,6 +59,7 @@ conf_defaults = {
     }
 
 conf_dotted = set([
+    'kotti.templates.api',
     'kotti.configurators',
     'kotti.base_includes',
     'kotti.includes',
@@ -125,25 +128,31 @@ def main(global_config, **settings):
         )
 
     config.begin()
-    from kotti.resources import appmaker
-    engine = engine_from_config(settings, 'sqlalchemy.')
-    config._set_root_factory(appmaker(engine))
+    config.commit()
 
-    _configure_base_views(config)
-
-    # Include modules listed in 'kotti.includes' and 'kotti.includes':
+    # Include modules listed in 'kotti.base_includes' and 'kotti.includes':
     for module in (
         settings['kotti.base_includes'] + settings['kotti.includes']):
         config.include(module)
 
+    from kotti.resources import appmaker
+    engine = engine_from_config(settings, 'sqlalchemy.')
+    config._set_root_factory(appmaker(engine))
+
+    import kotti.views.util
+    config.add_subscriber(
+        kotti.views.util.add_renderer_globals, BeforeRender)
+    
+    _configure_base_views(config)
+
     return config.make_wsgi_app()
 
 def _configure_base_views(config):
-    from kotti.resources import Node
+    from kotti.resources import IContent
 
     config.add_static_view('static-deform', 'deform:static')
     config.add_static_view('static-kotti', 'kotti:static')
-    config.add_view('kotti.views.view.view_node_default', context=Node)
+    config.add_view('kotti.views.view.view_content_default', context=IContent)
     config.add_view(
         'kotti.views.edit.add_node',
         name='add',
