@@ -1,6 +1,7 @@
+import os
 from UserDict import DictMixin
 
-import transaction
+from pyramid.traversal import resource_path
 from sqlalchemy.orm import backref
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import object_mapper
@@ -17,7 +18,9 @@ from sqlalchemy import Integer
 from sqlalchemy import String
 from sqlalchemy import Unicode
 from sqlalchemy import UnicodeText
-from pyramid.traversal import resource_path
+import transaction
+from zope.interface import implements
+from zope.interface import Interface
 
 from kotti import get_version
 from kotti import get_settings
@@ -25,6 +28,7 @@ from kotti import DBSession
 from kotti import metadata
 from kotti.util import ViewLink
 from kotti.util import JsonType
+from kotti.util import MutationDict
 from kotti.security import PersistentACL
 from kotti.security import get_principals
 from kotti.security import view_permitted
@@ -56,55 +60,23 @@ class Container(object, DictMixin):
     def keys(self):
         return [child.name for child in self.children]
 
-class TypeInfo(object):
-    addable_to = ()
+class INode(Interface):
+    pass
 
-    def __init__(self, **kwargs):
-        self.__dict__.update(kwargs)
-
-    def copy(self, **kwargs):
-        d = self.__dict__.copy()
-        d.update(kwargs)
-        return TypeInfo(**d)
-
-    def addable(self, context, request):
-        """Return True if the type described in 'self' may be added to
-        'context'.
-        """
-        if view_permitted(context, request, self.add_view):
-            return context.type_info.name in self.addable_to
-        else:
-            return False
+class IContent(Interface):
+    pass
 
 class Node(Container, PersistentACL):
-    type_info = TypeInfo(
-        name=u'Node',
-        add_view=None,
-        addable_to=[],
-        edit_links=[
-            ViewLink('edit'),
-            ViewLink('add'),
-            ViewLink('move'),
-            ViewLink('share'),
-            ],
-        )
+    implements(INode)
 
     id = None
-    def __init__(self, name=None, parent=None, default_view=None,
-                 title=u"", description=u"", language=None,
-                 owner=None, creation_date=None, modification_date=None,
-                 in_navigation=True):
+    def __init__(self, name=None, parent=None, title=u"", annotations=None):
+        if annotations is None:
+            annotations = {}
         self.name = name
         self.parent = parent
-        self.default_view = default_view
         self.title = title
-        self.description = description
-        self.language = language
-        self.owner = owner
-        # These are set by events if not defined at this point:
-        self.creation_date = creation_date
-        self.modification_date = modification_date
-        self.in_navigation = in_navigation
+        self.annotations = annotations
 
     # Provide location-awareness through __name__ and __parent__
     @property
@@ -125,17 +97,80 @@ class Node(Container, PersistentACL):
     def copy(self, **kwargs):
         copy = self.__class__()
         for prop in object_mapper(self).iterate_properties:
-            if prop.key not in ('id', 'parent', 'children'):
+            if prop.key not in ('id', 'parent', 'children', 'local_groups'):
                 setattr(copy, prop.key, getattr(self, prop.key))
         for key, value in kwargs.items():
             setattr(copy, key, value)
         children = list(self.children)
         for child in children:
             copy.children.append(child.copy())
+        for group in self.local_groups:
+            group.copy(node=copy)
         return copy
 
-class Document(Node):
-    type_info = Node.type_info.copy(
+class LocalGroup(object):
+    def __init__(self, node, principal_name, group_name):
+        self.node = node
+        self.principal_name = principal_name
+        self.group_name = group_name
+
+    def copy(self, **kwargs):
+        kwargs.setdefault('node', self.node)
+        kwargs.setdefault('principal_name', self.principal_name)
+        kwargs.setdefault('group_name', self.group_name)
+        return self.__class__(**kwargs)
+
+class TypeInfo(object):
+    addable_to = ()
+
+    def __init__(self, **kwargs):
+        self.__dict__.update(kwargs)
+
+    def copy(self, **kwargs):
+        d = self.__dict__.copy()
+        d.update(kwargs)
+        return TypeInfo(**d)
+
+    def addable(self, context, request):
+        """Return True if the type described in 'self' may be added to
+        'context'.
+        """
+        if view_permitted(context, request, self.add_view):
+            return context.type_info.name in self.addable_to
+        else:
+            return False
+
+class Content(Node):
+    implements(IContent)
+
+    type_info = TypeInfo(
+        name=u'Content',
+        add_view=None,
+        addable_to=[],
+        edit_links=[
+            ViewLink('edit'),
+            ViewLink('add'),
+            ViewLink('move'),
+            ViewLink('share'),
+            ],
+        )
+
+    def __init__(self, name=None, parent=None, title=u"", annotations=None,
+                 default_view=None, description=u"", language=None,
+                 owner=None, creation_date=None, modification_date=None,
+                 in_navigation=True):
+        super(Content, self).__init__(name, parent, title, annotations)
+        self.default_view = default_view
+        self.description = description
+        self.language = language
+        self.owner = owner
+        self.in_navigation = in_navigation
+        # These are set by events if not defined at this point:
+        self.creation_date = creation_date
+        self.modification_date = modification_date
+
+class Document(Content):
+    type_info = Content.type_info.copy(
         name=u'Document',
         add_view=u'add_document',
         addable_to=[u'Document'],
@@ -148,27 +183,40 @@ class Document(Node):
 
 nodes = Table('nodes', metadata,
     Column('id', Integer, primary_key=True),
-    Column('parent_id', ForeignKey('nodes.id')),
-    Column('name', Unicode(50), nullable=False),
     Column('type', String(30), nullable=False),
-    Column('_acl', JsonType()),
-    Column('__roles__', JsonType()),
-    Column('default_view', String(50)),
+    Column('parent_id', ForeignKey('nodes.id')),
     Column('position', Integer),
+    Column('_acl', JsonType()),
 
+    Column('name', Unicode(50), nullable=False),
     Column('title', Unicode(100)),
+    Column('annotations', MutationDict.as_mutable(JsonType)),
+
+    UniqueConstraint('parent_id', 'name'),
+)
+
+local_groups_table = Table('local_groups', metadata,
+    Column('id', Integer, primary_key=True),
+    Column('node_id', ForeignKey('nodes.id')),
+    Column('principal_name', Unicode(100)),
+    Column('group_name', Unicode(100)),
+
+    UniqueConstraint('node_id', 'principal_name', 'group_name'),
+)
+
+contents = Table('contents', metadata,
+    Column('id', Integer, ForeignKey('nodes.id'), primary_key=True),
+    Column('default_view', String(50)),
     Column('description', UnicodeText()),
     Column('language', Unicode(10)),
     Column('owner', Unicode(100)),
     Column('creation_date', DateTime()),
     Column('modification_date', DateTime()),
     Column('in_navigation', Boolean()),
-
-    UniqueConstraint('parent_id', 'name'),
 )
 
 documents = Table('documents', metadata,
-    Column('id', Integer, ForeignKey('nodes.id'), primary_key=True),
+    Column('id', Integer, ForeignKey('contents.id'), primary_key=True),
     Column('body', UnicodeText()),
     Column('mime_type', String(30)),
 )
@@ -187,10 +235,20 @@ mapper(
             backref=backref('parent', remote_side=[nodes.c.id]),
             cascade='all',
             ),
+        'local_groups': relation(
+            LocalGroup,
+            backref=backref('node'),
+            cascade='all',
+            )
         },
     )
 
-mapper(Document, documents, inherits=Node, polymorphic_identity='document')
+mapper(LocalGroup, local_groups_table)
+
+mapper(Content, contents, inherits=Node, polymorphic_identity='content')
+       
+mapper(Document, documents, inherits=Content, polymorphic_identity='document')
+
 
 class Settings(object):
     def __init__(self, data):
@@ -239,11 +297,13 @@ def populate():
     transaction.commit()
 
 _session = []
-def initialize_sql(engine):
+def initialize_sql(engine, drop_all=False):
     if _session:
         return _session[0]
     DBSession.configure(bind=engine)
     metadata.bind = engine
+    if drop_all or os.environ.get('KOTTI_TEST_DB_STRING'):
+        metadata.drop_all(engine) # pragma: no coverage
     metadata.create_all(engine)
     for populate in get_settings()['kotti.populators']:
         populate()
