@@ -1,7 +1,7 @@
 from datetime import datetime
-import hashlib
 from UserDict import DictMixin
 
+import bcrypt
 from sqlalchemy import Column
 from sqlalchemy import Boolean
 from sqlalchemy import Integer
@@ -15,6 +15,7 @@ from pyramid.location import lineage
 from pyramid.security import Allow
 from pyramid.security import ALL_PERMISSIONS
 from pyramid.security import view_execution_permitted
+from pyramid.security import authenticated_userid
 
 from kotti import get_settings
 from kotti import DBSession
@@ -54,6 +55,7 @@ class Principal(object):
         self.email = email
         self.groups = groups
         self.creation_date = datetime.now()
+        self.last_login_date = None
 
     def __repr__(self): # pragma: no cover
         return '<Principal %r>' % self.name
@@ -64,8 +66,12 @@ class AbstractPrincipals(object):
 
     Principals mostly provides dict-like access to the principal
     objects in the database.  In addition, there's the 'search' method
-    which allows searching users and groups, and the 'hash_password'
-    method that implements user password hashing.
+    which allows searching users and groups.
+
+    'hash_password' is for initial hashing of a clear text password,
+    while 'validate_password' is used by the login to see if the
+    entered password matches the hashed password that's already in the
+    database.
 
     Use the 'kotti.principals' settings variable to override Kotti's
     default Principals implementation with your own.
@@ -91,8 +97,8 @@ class AbstractPrincipals(object):
         """
 
     def search(self, **kwargs):
-        """Return a list of principal objects that correspond to the
-        search arguments passed in.
+        """Return an iterable with principal objects that correspond
+        to the search arguments passed in.
 
         This example would return all principals with the id 'bob':
 
@@ -117,6 +123,10 @@ class AbstractPrincipals(object):
         This is what's stored in the database as 'principal.password'.
         """
 
+    def validate_password(self, clear, hashed):
+        """Returns True if the clear text password matches the hash.
+        """
+
 ROLES = {
     u'role:viewer': Principal(u'role:viewer', title=u'Viewer'),
     u'role:editor': Principal(u'role:editor', title=u'Editor'),
@@ -180,15 +190,17 @@ class PersistentACL(object):
             (Allow, 'role:admin', ALL_PERMISSIONS),
             ]
 
-def all_groups_raw(context):
-    return getattr(context, '__roles__', None)
-
 def list_groups_raw(name, context):
-    groups = all_groups_raw(context)
-    if groups is not None:
-        return groups.get(name, set())
-    else:
-        return set()
+    """A set of group names in given ``context`` for ``name``.
+
+    Only groups defined in context will be considered, therefore no
+    global or inherited groups are returned.
+    """
+    from kotti.resources import LocalGroup
+    session = DBSession()
+    return set(r[0] for r in session.query(LocalGroup.group_name).filter(
+        LocalGroup.node_id==context.id).filter(
+        LocalGroup.principal_name==name).all())
 
 def list_groups(name, context=None):
     """List groups for principal with a given ``name``.
@@ -207,6 +219,7 @@ def _cachekey_list_groups_ext(name, context=None, _seen=None, _inherited=None):
 
 @request_cache(_cachekey_list_groups_ext)
 def list_groups_ext(name, context=None, _seen=None, _inherited=None):
+    name = unicode(name)
     groups = set()
     recursing = _inherited is not None
     _inherited = _inherited or set()
@@ -241,23 +254,19 @@ def list_groups_ext(name, context=None, _seen=None, _inherited=None):
 
     return list(groups), list(_inherited)
 
-def set_groups_raw(context, groups):
-    context.__roles__ = groups
-
-def set_groups(name, context, groups_to_set):
+def set_groups(name, context, groups_to_set=()):
     """Set the list of groups for principal with given ``name`` and in
     given ``context``.
     """
-    groups = all_groups_raw(context)
-    if groups is None:
-        groups = {}
-    else:
-        groups = dict(groups)
-    if groups_to_set:
-        groups[name] = list(groups_to_set)
-    else:
-        groups.pop(name, None)
-    set_groups_raw(context, groups)
+    name = unicode(name)
+    from kotti.resources import LocalGroup
+    session = DBSession()
+    session.query(LocalGroup).filter(
+        LocalGroup.node_id==context.id).filter(
+        LocalGroup.principal_name==name).delete()
+
+    for group_name in groups_to_set:
+        session.add(LocalGroup(context, name, unicode(group_name)))
 
 def list_groups_callback(name, request):
     if not is_user(name):
@@ -278,16 +287,24 @@ def view_permitted(context, request, name=''):
     finally:
         del request.environ['authz_context']
 
-def principals_with_local_roles(context):
-    """Return a list of principal names that have local roles
-    (inherited or not) in the context.
+def principals_with_local_roles(context, inherit=True):
+    """Return a list of principal names that have local roles in the
+    context.
     """
+    from resources import LocalGroup
+    session = DBSession()
     principals = set()
-    for item in lineage(context):
-        agr = all_groups_raw(item)
-        if agr is not None:
-            ap = [p for p in agr.keys() if not p.startswith('role:')]
-            principals.update(ap)
+    items = [context]
+    if inherit:
+        items = lineage(context)
+    for item in items:
+        principals.update(
+            r[0] for r in
+            session.query(LocalGroup.principal_name).filter(
+                LocalGroup.node_id==item.id).group_by(
+                LocalGroup.principal_name).all()
+            if not r[0].startswith('role:')
+            )
     return list(principals)
 
 def map_principals_with_local_roles(context):
@@ -299,8 +316,7 @@ def map_principals_with_local_roles(context):
         except KeyError:
             continue
         else:
-            all, inherited = list_groups_ext(
-                principal_name, context)
+            all, inherited = list_groups_ext(principal_name, context)
             value.append((principal, (all, inherited)))
     return sorted(value, key=lambda t: t[0].name)
 
@@ -312,6 +328,10 @@ def is_user(principal):
         principal = principal.name
     return ':' not in principal
 
+def get_user(request):
+    userid = authenticated_userid(request)
+    return get_principals().get(userid)
+
 class Principals(DictMixin):
     """Kotti's default principal database.
 
@@ -374,9 +394,17 @@ class Principals(DictMixin):
         query = query.filter(or_(*filters))
         return query
 
-    def hash_password(self, password):
-        salt = get_settings()['kotti.secret']
-        return unicode(hashlib.sha224(salt + password).hexdigest())
+    log_rounds = 10
+    def hash_password(self, password, hashed=None):
+        if hashed is None:
+            hashed = bcrypt.gensalt(self.log_rounds)
+        return unicode(bcrypt.hashpw(password, hashed))
+
+    def validate_password(self, clear, hashed):
+        try:
+            return self.hash_password(clear, hashed) == hashed
+        except ValueError:
+            return False
 
 def principals_factory():
     return Principals()
@@ -391,6 +419,7 @@ principals_table = Table('principals', metadata,
     Column('email', Unicode(100), unique=True),
     Column('groups', JsonType(), nullable=False),
     Column('creation_date', DateTime(), nullable=False),
+    Column('last_login_date', DateTime()),
 )
 
 mapper(Principal, principals_table, order_by=principals_table.c.name)
