@@ -1,4 +1,5 @@
 from contextlib import contextmanager
+import os
 import unittest
 
 import transaction
@@ -19,7 +20,9 @@ from kotti import DBSession
 from kotti.events import clear
 from kotti.message import _inject_mailer
 from kotti.resources import Node
+from kotti.resources import Content
 from kotti.resources import Document
+from kotti.resources import LocalGroup
 from kotti.resources import initialize_sql
 from kotti.resources import get_root
 from kotti.security import list_groups
@@ -41,20 +44,29 @@ BASE_URL = 'http://localhost:6543'
 class DummyRequest(testing.DummyRequest):
     is_xhr = False
 
+    def is_response(self, ob):
+        if ( hasattr(ob, 'app_iter') and hasattr(ob, 'headerlist') and
+             hasattr(ob, 'status') ):
+            return True
+        return False
+
+def testing_db_url():
+    return os.environ.get('KOTTI_TEST_DB_STRING', 'sqlite://')
+
 def _initTestingDB():
     from sqlalchemy import create_engine
-    session = initialize_sql(create_engine('sqlite://'))
+    database_url = testing_db_url()
+    session = initialize_sql(create_engine(database_url), drop_all=True)
     return session
 
 def setUp(**kwargs):
     tearDown()
-    settings = kwargs.get('settings', {})
-    if not settings:
-        settings = conf_defaults.copy()
-        settings['kotti.secret'] = 'secret'
-        settings['kotti.secret2'] = 'secret2'
-        _resolve_dotted(settings)
-        kwargs['settings'] = settings
+    settings = conf_defaults.copy()
+    settings['kotti.secret'] = 'secret'
+    settings['kotti.secret2'] = 'secret2'
+    _resolve_dotted(settings)
+    settings.update(kwargs.get('settings', {}))
+    kwargs['settings'] = settings
     config = testing.setUp(**kwargs)
 
     _initTestingDB()
@@ -216,15 +228,26 @@ class TestNode(UnitTestBase):
         self.assertEqual(copy_of_root.name, u'copy_of_root')
         self.assertEqual(root.name, u'')
 
+    def test_annotations_mutable(self):
+        root = get_root()
+        session = DBSession()
+        root.annotations['foo'] = u'bar'
+        self.assertTrue(root in session.dirty)
+        del root.annotations['foo']
+
+    def test_annotations_coerce_fail(self):
+        root = get_root()
+        self.assertRaises(ValueError, setattr, root, 'annotations', [])
+
 class TestSecurity(UnitTestBase):
     def test_root_default(self):
         root = get_root()
         self.assertEqual(list_groups('admin', root), ['role:admin'])
-        self.assertEqual(list_groups_raw('admin', root), set([]))
+        self.assertEqual(list_groups_raw(u'admin', root), set([]))
 
     def test_empty(self):
         root = get_root()
-        self.assertEqual(list_groups(root, 'bob'), [])
+        self.assertEqual(list_groups('bob', root), [])
 
     def test_simple(self):
         root = get_root()
@@ -232,7 +255,26 @@ class TestSecurity(UnitTestBase):
         self.assertEqual(
             list_groups('bob', root), ['role:editor'])
         self.assertEqual(
-            list_groups_raw('bob', root), ['role:editor'])
+            list_groups_raw(u'bob', root), set(['role:editor']))
+
+    def test_overwrite_and_delete(self):
+        root = get_root()
+        set_groups('bob', root, ['role:editor'])
+        self.assertEqual(
+            list_groups('bob', root), ['role:editor'])
+        self.assertEqual(
+            list_groups_raw(u'bob', root), set(['role:editor']))
+
+        set_groups('bob', root, ['role:admin'])
+        self.assertEqual(
+            list_groups('bob', root), ['role:admin'])
+        self.assertEqual(
+            list_groups_raw(u'bob', root), set(['role:admin']))
+
+        set_groups('bob', root)
+        self.assertEqual(
+            list_groups('bob', root), [])
+        self.assertTrue(get_root() is root)
 
     def test_inherit(self):
         session = DBSession()
@@ -253,7 +295,7 @@ class TestSecurity(UnitTestBase):
 
         # We can ask to list only those groups that are defined locally:
         self.assertEqual(
-            list_groups_raw('bob', child), ['group:somegroup'])
+            list_groups_raw(u'bob', child), set(['group:somegroup']))
 
     @staticmethod
     def add_some_groups():
@@ -442,11 +484,40 @@ class TestSecurity(UnitTestBase):
             set(principals_with_local_roles(child)),
             set(['bob', 'group:bobsgroup', 'group:franksgroup'])
             )
+        self.assertEqual(
+            set(principals_with_local_roles(child, inherit=False)),
+            set(['group:bobsgroup'])
+            )
         self.assertEqual(
             set(principals_with_local_roles(root)),
             set(['bob', 'group:franksgroup'])
             )
 
+    def test_copy_local_groups(self):
+        self.test_principals_with_local_roles()
+        root = get_root()
+        child = root[u'child']
+        self.assertEqual(
+            set(principals_with_local_roles(child)),
+            set(['bob', 'group:bobsgroup', 'group:franksgroup'])
+            )
+
+        # We make a copy of 'child', and we expect the local roles to
+        # be copied over:
+        child2 = root['child2'] = child.copy()
+        DBSession.flush()
+        self.assertEqual(
+            set(principals_with_local_roles(child2)),
+            set(['bob', 'group:bobsgroup', 'group:franksgroup'])
+            )
+        self.assertEqual(len(principals_with_local_roles(child)), 3)
+
+        # When we now change the local roles of 'child', the copy is
+        # unaffected:
+        set_groups('group:bobsgroup', child, [])
+        self.assertEqual(len(principals_with_local_roles(child)), 2)
+        self.assertEqual(len(principals_with_local_roles(child2)), 3)
+
     def test_map_principals_with_local_roles(self):
         self.test_principals_with_local_roles()
         root = get_root()
@@ -476,6 +547,22 @@ class TestSecurity(UnitTestBase):
         self.assertEqual(bobsgroup_all, ['role:editor'])
         self.assertEqual(bobsgroup_inherited, [])
 
+    def test_local_roles_db_cascade(self):
+        session = DBSession()
+        root = get_root()
+        child = root[u'child'] = Node()
+        session.flush()
+
+        # We set a local group on child and delete child.  We then
+        # expect the LocalGroup entry to have been deleted from the
+        # database:
+        self.assertEqual(session.query(LocalGroup).count(), 0)
+        set_groups('group:bobsgroup', child, ['role:editor'])
+        self.assertEqual(session.query(LocalGroup).count(), 1)
+        del root[u'child']
+        session.flush()
+        self.assertEqual(session.query(LocalGroup).count(), 0)
+
 class TestPrincipals(UnitTestBase):
     def get_principals(self):
         return get_principals()
@@ -498,8 +585,8 @@ class TestPrincipals(UnitTestBase):
 
     def test_default_admin(self):
         admin = self.get_principals()[u'admin']
-        hashed = self.get_principals().hash_password(u'secret')
-        self.assertEqual(admin.password, hashed)
+        self.assertTrue(
+            self.get_principals().validate_password(u'secret', admin.password))
         self.assertEqual(admin.groups, [u'role:admin'])
 
     def test_users_empty(self):
@@ -530,7 +617,7 @@ class TestPrincipals(UnitTestBase):
         self._assert_is_bob(bob)
         self.assertRaises(AttributeError,
                           users.search, name=u"bob", foo=u"bar")
-        self.assertEqual(users.search(), [])
+        self.assertEqual(list(users.search()), [])
 
     def test_groups_from_users(self):
         self.make_bob()
@@ -560,28 +647,28 @@ class TestPrincipals(UnitTestBase):
         bob.name = u'group:bobsgroup'
         self.assertEqual(is_user(bob), False)
 
-    def test_hash_password(self):
+    def test_hash_and_validate_password(self):
         password = u"secret"
-        hash_password = self.get_principals().hash_password
-
-        # For 'hash_password' to work, we need to set a secret:
-        settings = self.config.registry.settings
-        settings['kotti.secret'] = 'there is no secret'
-        hashed = hash_password(password)
-        self.assertEqual(hashed, hash_password(password))
-        settings['kotti.secret'] = 'different'
-        self.assertNotEqual(hashed, hash_password(password))        
-        del settings['kotti.secret']
+        principals = self.get_principals()
+        hashed = principals.hash_password(password)
+        self.assertTrue(principals.validate_password(password, hashed))
+        self.assertFalse(principals.validate_password(u"foo", hashed))
 
     def test_bobs_hashed_password(self):
         bob = self.make_bob()
-        hash_password = self.get_principals().hash_password
-        self.assertEqual(bob.password, hash_password(u'secret'))
+        principals = self.get_principals()
+        self.assertTrue(principals.validate_password(u"secret", bob.password))
 
         # When we set the 'password' attribute directly, we have to do
-        # the hashing ourselves.
-        bob.password = u'hashed secret'
-        self.assertEqual(bob.password, u'hashed secret')
+        # the hashing ourselves.  This won't work:
+        bob.password = u'anothersecret'
+        self.assertFalse(
+            principals.validate_password(u"anothersecret", bob.password))
+
+        # This will:
+        bob.password = principals.hash_password(u"anothersecret")
+        self.assertTrue(
+            principals.validate_password(u"anothersecret", bob.password))
 
     def test_active(self):
         bob = self.make_bob()
@@ -611,10 +698,13 @@ class TestPrincipals(UnitTestBase):
 
         # Make Bob and do it again:
         bob = self.make_bob()
+        self.assertEqual(bob.last_login_date, None)
         result = login(None, request)
         self.assertEqual(result.status, '302 Found')
         self.assertEqual(request.session.pop_flash('success'),
                          [u'Welcome, Bob Dabolina!'])
+        last_login_date = bob.last_login_date
+        self.assertNotEqual(last_login_date, None)
         self.assertEqual(request.session.pop_flash('error'), [])
 
         # Log in with email:
@@ -623,6 +713,7 @@ class TestPrincipals(UnitTestBase):
         self.assertEqual(result.status, '302 Found')
         self.assertEqual(request.session.pop_flash('success'),
                          [u'Welcome, Bob Dabolina!'])
+        self.assertTrue(last_login_date < bob.last_login_date)
 
         # Deactive Bob, logging in is no longer possible:
         bob.active = False
@@ -654,39 +745,31 @@ class TestEvents(UnitTestBase):
         session = DBSession()
         self.config.testing_securitypolicy(userid=u'bob')
         root = get_root()
-        child = root[u'child'] = Node()
+        child = root[u'child'] = Content()
         session.flush()
         self.assertEqual(child.owner, u'bob')
         self.assertEqual(list_groups(u'bob', child), [u'role:owner'])
 
         clear_request_cache()
         # The event listener does not set the role again for subitems:
-        grandchild = child[u'grandchild'] = Node()
+        grandchild = child[u'grandchild'] = Content()
         session.flush()
         self.assertEqual(grandchild.owner, u'bob')
         self.assertEqual(list_groups(u'bob', grandchild), [u'role:owner'])
         self.assertEqual(len(list_groups_raw(u'bob', grandchild)), 0)
 
-class TestNodeView(UnitTestBase):
-    def test_it(self):
-        from kotti.views.view import view_node
-        root = get_root()
-        request = DummyRequest()
-        info = view_node(root, request)
-        self.assertEqual(info['api'].context, root)
-
 @contextmanager
-def nodes_addable():
+def contents_addable():
     # Allow Nodes to be added to documents:
-    save_node_type_info = Node.type_info.copy()
-    Node.type_info.addable_to = [u'Document']
-    Node.type_info.add_view = u'add_document'
-    get_settings()['kotti.available_types'].append(Node)
+    save_node_type_info = Content.type_info.copy()
+    Content.type_info.addable_to = [u'Document']
+    Content.type_info.add_view = u'add_document'
+    get_settings()['kotti.available_types'].append(Content)
     try:
         yield
     finally:
         get_settings()['kotti.available_types'].pop()
-        Node.type_info = save_node_type_info
+        Content.type_info = save_node_type_info
 
 class TestAddableTypes(UnitTestBase):
     def test_view_permitted_yes(self):
@@ -709,15 +792,16 @@ class TestAddableTypes(UnitTestBase):
         root = get_root()
         request = DummyRequest()
 
-        with nodes_addable():
+        with contents_addable():
             # We should be able to add both Nodes and Documents now:
             possible_parents, possible_types = addable_types(root, request)
             self.assertEqual(len(possible_parents), 1)
-            self.assertEqual(possible_parents[0]['factories'], [Document, Node])
+            self.assertEqual(possible_parents[0]['factories'],
+                             [Document, Content])
 
             document_info, node_info = possible_types
             self.assertEqual(document_info['factory'], Document)
-            self.assertEqual(node_info['factory'], Node)
+            self.assertEqual(node_info['factory'], Content)
             self.assertEqual(document_info['nodes'], [root])
             self.assertEqual(node_info['nodes'], [root])
 
@@ -727,19 +811,19 @@ class TestAddableTypes(UnitTestBase):
         root = get_root()
         request = DummyRequest()
 
-        with nodes_addable():
+        with contents_addable():
             # We should be able to add both to the child and to the parent:
             child = root['child'] = Document(title=u"Child")
             possible_parents, possible_types = addable_types(child, request)
             child_parent, root_parent = possible_parents
             self.assertEqual(child_parent['node'], child)
             self.assertEqual(root_parent['node'], root)
-            self.assertEqual(child_parent['factories'], [Document, Node])
-            self.assertEqual(root_parent['factories'], [Document, Node])
+            self.assertEqual(child_parent['factories'], [Document, Content])
+            self.assertEqual(root_parent['factories'], [Document, Content])
 
             document_info, node_info = possible_types
             self.assertEqual(document_info['factory'], Document)
-            self.assertEqual(node_info['factory'], Node)
+            self.assertEqual(node_info['factory'], Content)
             self.assertEqual(document_info['nodes'], [child, root])
             self.assertEqual(node_info['nodes'], [child, root])
 
@@ -765,7 +849,7 @@ class TestNodeEdit(UnitTestBase):
         root = get_root()
         request = DummyRequest()
 
-        with nodes_addable():
+        with contents_addable():
             # The child Document does not contain any other Nodes, so it's
             # second in the 'possible_parents' list returned by 'node_add':
             child = root['child'] = Document(title=u"Child")
@@ -989,15 +1073,15 @@ class TestUserManagement(UnitTestBase):
 
 class TestTemplateAPI(UnitTestBase):
     def _make(self, context=None, request=None, id=1, **kwargs):
-        from kotti.views.util import TemplateAPIEdit
+        from kotti.views.util import TemplateAPI
         if context is None:
             session = DBSession()
             context = session.query(Node).get(id)
         if request is None:
             request = DummyRequest()
-        return TemplateAPIEdit(context, request, **kwargs)
+        return TemplateAPI(context, request, **kwargs)
 
-    def _create_nodes(self, root):
+    def _create_contents(self, root):
         # root -> a --> aa
         #         |
         #         \ --> ab
@@ -1005,21 +1089,23 @@ class TestTemplateAPI(UnitTestBase):
         #         \ --> ac --> aca
         #               |
         #               \ --> acb
-        a = root['a'] = Node()
-        aa = root['a']['aa'] = Node()
-        ab = root['a']['ab'] = Node()
-        ac = root['a']['ac'] = Node()
-        aca = ac['aca'] = Node()
-        acb = ac['acb'] = Node()
+        a = root['a'] = Content()
+        aa = root['a']['aa'] = Content()
+        ab = root['a']['ab'] = Content()
+        ac = root['a']['ac'] = Content()
+        aca = ac['aca'] = Content()
+        acb = ac['acb'] = Content()
         return a, aa, ab, ac, aca, acb
 
     def test_page_title(self):
-        from kotti.views.util import TemplateAPI
-        edit_api = self._make()
-        view_api = TemplateAPI(edit_api.context, edit_api.request)
-        view_api.root.title = u"Hello, world!"
-        self.assertEqual(edit_api.page_title, u" - Hello, world!")
-        self.assertEqual(view_api.page_title, u"Hello, world! - Hello, world!")
+        api = self._make()
+        api.context.title = u"Hello, world!"
+        self.assertEqual(api.page_title, u"Hello, world! - Hello, world!")
+
+        api = self._make()
+        api.context.title = u"Hello, world!"
+        api.site_title = u"Wasnhierlos"
+        self.assertEqual(api.page_title, u"Hello, world! - Wasnhierlos")
 
     def test_list_children(self):
         api = self._make() # the default context is root
@@ -1027,7 +1113,7 @@ class TestTemplateAPI(UnitTestBase):
         self.assertEquals(len(api.list_children(root)), 0)
 
         # Now try it on a little graph:
-        a, aa, ab, ac, aca, acb = self._create_nodes(root)
+        a, aa, ab, ac, aca, acb = self._create_contents(root)
         self.assertEquals(api.list_children(root), [a])
         self.assertEquals(api.list_children(a), [aa, ab, ac])
         self.assertEquals(api.list_children(aca), [])
@@ -1042,7 +1128,7 @@ class TestTemplateAPI(UnitTestBase):
     def test_root(self):
         api = self._make()
         root = api.context
-        a, aa, ab, ac, aca, acb = self._create_nodes(root)
+        a, aa, ab, ac, aca, acb = self._create_contents(root)
         self.assertEquals(self._make().root, root)
         self.assertEquals(self._make(acb).root, root)
 
@@ -1076,7 +1162,7 @@ class TestTemplateAPI(UnitTestBase):
         # 'context_links' returns a two-tuple of the form (siblings,
         # children), where the URLs point to edit pages:
         root = self._make().root
-        a, aa, ab, ac, aca, acb = self._create_nodes(root)
+        a, aa, ab, ac, aca, acb = self._create_contents(root)
         api = self._make(ac)
         siblings, children = api.context_links
 
@@ -1092,7 +1178,7 @@ class TestTemplateAPI(UnitTestBase):
 
     def test_breadcrumbs(self):
         root = self._make().root
-        a, aa, ab, ac, aca, acb = self._create_nodes(root)
+        a, aa, ab, ac, aca, acb = self._create_contents(root)
         api = self._make(acb)
         breadcrumbs = api.breadcrumbs
         self.assertEqual(
@@ -1147,7 +1233,8 @@ class TestTemplateAPI(UnitTestBase):
         api = self._make()
         self.assertEqual(
             sorted(api.slots.keys()),
-            ['abovecontent', 'belowcontent', 'left', 'right']
+            ['abovecontent', 'beforebodyend', 'belowcontent', 'inhead',
+             'left', 'right']
             )
         for key in api.slots.keys():
             self.assertEqual(api.slots[key], [])
@@ -1156,7 +1243,7 @@ class TestTemplateAPI(UnitTestBase):
         from kotti.views.slots import render_local_navigation
         root = DBSession().query(Node).get(1)
         request = DummyRequest()
-        a, aa, ab, ac, aca, acb = self._create_nodes(root)
+        a, aa, ab, ac, aca, acb = self._create_contents(root)
         self.assertEqual(render_local_navigation(root, request), None)
         self.assertNotEqual(render_local_navigation(a, request), None)
         self.assertEqual("ab" in render_local_navigation(a, request), True)
@@ -1214,6 +1301,54 @@ class TestTemplateAPI(UnitTestBase):
         api.locale_name = 'unknown'
         self.assertRaises(UnknownLocaleError, api.format_time, first)
 
+    def test_render_view(self):
+        from pyramid.response import Response
+        def first_view(context, request):
+            return Response(u'first')
+        def second_view(context, request):
+            return Response(u'second')
+        self.config.add_view(first_view, name='')
+        self.config.add_view(second_view, name='second')
+        api = self._make()
+        self.assertEqual(api.render_view(), u'first')
+        self.assertEqual(api.render_view('second'), u'second')
+        self.assertEqual(
+            api.render_view(context=api.context, request=api.request), u'first')
+
+    def test_get_type(self):
+        from kotti.resources import Document
+        api = self._make()
+        self.assertEqual(api.get_type('Document'), Document)
+        self.assertEqual(api.get_type('NoExist'), None)
+
+class TestViewUtil(UnitTestBase):
+    def test_add_renderer_globals_json(self):
+        from kotti.views.util import add_renderer_globals
+
+        event = {'renderer_name': 'json'}
+        add_renderer_globals(event)
+        self.assertEqual(event.keys(), ['renderer_name'])
+
+    def test_add_renderer_globals_request_has_template_api(self):
+        from kotti.views.util import add_renderer_globals
+
+        request = DummyRequest()
+        request.template_api = template_api = object()
+        event = {'request': request, 'renderer_name': 'foo'}
+        add_renderer_globals(event)
+        self.assertTrue(event['api'] is template_api)
+
+    def test_add_renderer_globals(self):
+        from kotti.views.util import add_renderer_globals
+
+        request = DummyRequest()
+        event = {
+            'request': request,
+            'context': object(),
+            'renderer_name': 'foo',
+            }
+        add_renderer_globals(event)
+        self.assertTrue('api' in event)
 
 class TestUtil(UnitTestBase):
     def test_title_to_name(self):
@@ -1281,7 +1416,7 @@ def setUpFunctional(global_config=None, **settings):
     import wsgi_intercept.zope_testbrowser
 
     _settings = {
-        'sqlalchemy.url': 'sqlite://',
+        'sqlalchemy.url': testing_db_url(),
         'kotti.secret': 'secret',
         'kotti.site_title': 'Website des Kottbusser Tors', # for mailing
         'mail.default_sender': 'kotti@localhost',
