@@ -1,9 +1,11 @@
 from pyramid.compat import json
+import string
 import urllib
 
 from pyramid.threadlocal import get_current_request
 from pyramid.url import resource_url
 from sqlalchemy.types import TypeDecorator, VARCHAR
+from sqlalchemy.ext.mutable import Mutable
 
 class JsonType(TypeDecorator):
     """http://www.sqlalchemy.org/docs/core/types.html#marshal-json-strings
@@ -13,7 +15,6 @@ class JsonType(TypeDecorator):
     def process_bind_param(self, value, dialect):
         if value is not None:
             value = json.dumps(value)
-
         return value
 
     def process_result_value(self, value, dialect):
@@ -21,6 +22,26 @@ class JsonType(TypeDecorator):
             value = json.loads(value)
         return value
 
+class MutationDict(Mutable, dict):
+    """http://www.sqlalchemy.org/docs/orm/extensions/mutable.html
+    """
+    @classmethod
+    def coerce(cls, key, value):
+        if not isinstance(value, MutationDict):
+            if isinstance(value, dict):
+                return MutationDict(value)
+            return Mutable.coerce(key, value)
+        else:
+            return value
+
+    def __setitem__(self, key, value):
+        dict.__setitem__(self, key, value)
+        self.changed()
+
+    def __delitem__(self, key):
+        dict.__delitem__(self, key)
+        self.changed()
+
 class ViewLink(object):
     def __init__(self, path, title=None):
         self.path = path
@@ -97,3 +118,9 @@ def extract_from_settings(prefix):
         if key.startswith(prefix):
             extracted[key[len(prefix):]] = value
     return extracted
+
+def title_to_name(title):
+    okay = string.letters + string.digits + '-'
+    name = u'-'.join(title.lower().split())
+    name = u''.join(ch for ch in name if ch in okay)
+    return name
