@@ -1,5 +1,4 @@
 import hashlib
-import string
 import urllib
 
 from babel.dates import format_date
@@ -11,26 +10,42 @@ from pyramid.i18n import get_locale_name
 from pyramid.location import inside
 from pyramid.location import lineage
 from pyramid.renderers import get_renderer
-from pyramid.security import authenticated_userid
 from pyramid.security import has_permission
 from pyramid.url import resource_url
+from pyramid.view import render_view_to_response
 from deform import ValidationFailure
 
 from kotti import get_settings
 from kotti import DBSession
+from kotti.util import title_to_name
 from kotti.events import objectevent_listeners
 from kotti.resources import Node
-from kotti.security import get_principals
 from kotti.security import view_permitted
+from kotti.security import get_user
 from kotti.views.slots import slot_events
 
+def template_api(context, request, **kwargs):
+    return get_settings()['kotti.templates.api'][0](
+        context, request, **kwargs)
+
+def render_view(context, request, name='', secure=True):
+    response = render_view_to_response(context, request, name, secure)
+    return response.ubody
+
+def add_renderer_globals(event):
+    if event['renderer_name'] != 'json':
+        api = getattr(event['request'], 'template_api', None)
+        if api is None:
+            api = template_api(event['context'], event['request'])
+        event['api'] = api
+
 class TemplateAPI(object):
     """This implements the 'api' object that's passed to all
     templates.
 
     Use dict-access as a shortcut to retrieve template macros from
-    templates.  ``api['master_view.messages']`` will return the
-    'messages' macro from the 'master_edit' template.
+    templates.  ``api['snippets.messages']`` will return the
+    'messages' macro from the 'snippets' template.
     """
     BARE_TMPL = 'kotti:templates/master-bare.pt'
 
@@ -53,9 +68,20 @@ class TemplateAPI(object):
         self.view_css = S['kotti.templates.view_css']
         self.edit_css = S['kotti.templates.edit_css']
 
+        if getattr(request, 'template_api', None) is None:
+            request.template_api = self
+
         self.__dict__.update(kwargs)
 
     def __getitem__(self, dottedname):
+        """Given a dottedname of the form ``template_name.macro_name``
+        this will return the ``macro_name`` macro of the
+        ``template_name`` template.
+
+        The template that corresponds to ``template_name`` may be
+        defined through the ``kotti.templates.*`` configuration
+        variables.
+        """
         try:
             template_name, macro_name = dottedname.split('.')
         except ValueError: # Chameleon will try dict access after attr access
@@ -73,7 +99,15 @@ class TemplateAPI(object):
 
     @reify
     def page_title(self):
-        return u'%s - %s' % (self.context.title, self.site_title)
+        view_title = self.request.view_name.replace('_', ' ').title()
+        if view_title:
+            view_title += u' '
+        view_title += self.context.title
+        return u'%s - %s' % (view_title, self.site_title)
+
+    @reify
+    def first_heading(self):
+        return u'<h1>%s</h1>' % self.page_title
 
     def url(self, context=None, *elements):
         if context is None:
@@ -91,14 +125,20 @@ class TemplateAPI(object):
 
     @reify
     def user(self):
-        userid = authenticated_userid(self.request)
-        return get_principals().get(userid)
+        return get_user(self.request)
     
     def has_permission(self, permission, context=None):
         if context is None:
             context = self.context
         return has_permission(permission, context, self.request)
 
+    def render_view(self, name='', context=None, request=None, secure=True):
+        if context is None:
+            context = self.context
+        if request is None:
+            request = self.request
+        return render_view(context, request, name, secure)
+
     def list_children(self, context=None, permission='view'):
         if context is None:
             context = self.context
@@ -166,15 +206,10 @@ class TemplateAPI(object):
             format = self.S['kotti.time_format']
         return format_time(t, format=format, locale=self.locale_name)
 
-class TemplateAPIEdit(TemplateAPI):
-    @reify
-    def page_title(self):
-        return u'%s - %s' % (
-            self.request.view_name.replace('_', ' ').title(), self.site_title)
-
-    @reify
-    def first_heading(self):
-        return u'<h1>Edit <em>%s</em></h1>' % self.context.title
+    def get_type(self, name):
+        for class_ in get_settings()['kotti.available_types']:
+            if class_.type_info.name == name:
+                return class_
 
     def _find_edit_view(self, item):
         view_name = self.request.view_name
@@ -272,12 +307,6 @@ def addable_types(context, request):
 
     return possible_parents, possible_types
 
-def title_to_name(title):
-    okay = string.letters + string.digits + '-'
-    name = u'-'.join(title.lower().split())
-    name = u''.join(ch for ch in name if ch in okay)
-    return name
-
 def disambiguate_name(name):
     parts = name.split(u'-')
     if len(parts) > 1:
@@ -342,9 +371,12 @@ class FormController(object):
     def appstruct(self, item):
         return item.__dict__.copy()
 
-    def edit_item(self, context, request, appstruct):
+    def edit(self, item, **appstruct):
         for key, value in appstruct.items():
-            setattr(context, key, value)
+            setattr(item, key, value)
+
+    def edit_item(self, context, request, appstruct):
+        self.edit(context, **appstruct)
         request.session.flash(self.edit_success_msg, 'success')
         try:
             location = resource_url(context, request) + self.success_path
@@ -353,9 +385,11 @@ class FormController(object):
         return HTTPFound(location=location)
         
     def add_item(self, context, request, appstruct):
-        name = title_to_name(appstruct['title'])
-        while name in context.keys():
-            name = disambiguate_name(name)
+        name = appstruct.get('name')
+        if name is None:
+            name = title_to_name(appstruct['title'])
+            while name in context.keys():
+                name = disambiguate_name(name)
         item = context[name] = self.add(**appstruct)
         request.session.flash(self.add_success_msg, 'success')
         location = resource_url(item, request) + self.success_path
